require 'net/http'
require 'uri'
require 'rubygems'
require 'json'

class AlterEgo
  
  API_URL = "https://alteregoapp.com/api"
  AUTH_URL = "http://alteregoapp.com/account/authorize-app"
  
  # Generate an AlterEgo +authorization_url+.
  # Returns a URL to redirect users to so that they will be prompted
  # to enter their AlterEgo username and password to authorize a
  # connection between your application and their AlterEgo account.
  #
  # If authorized successfully, a POST request will be sent to the
  # +redirect_url+ with a "key" parameter containing the API key for
  # that user's AlterEgo account. Be sure to store this key somewhere,
  # as you will need it to run API requests later.
  #
  # If authorization fails for some reason, an "error" parameter will
  # be present in the POST request, containing an error message.
  #
  # == A note about callback URL's:
  #
  # AlterEgo imposes the following requirements on the +redirect_url+ value
  # to ensure security. Be sure that your +redirect_url+ meets the following
  # requirements:
  #
  # - Must be served via HTTPS.
  # - Must be under the same domain (or a subdomain of) the website entered
  #   when registering your application with AlterEgo.
  #
  # == Example
  #
  # redirect_to AlterEgo.authorization_url("12345","https://example.com/callback")
  #
  def self.authorization_url(app_id, redirect_url)
    # TODO: Encode the redirect_url.
    "#{AUTH_URL}?id=#{app_id}&redirect_url=#{URI.escape(redirect_url, Regexp.new("[^#{URI::PATTERN::UNRESERVED}]"))}"
  end
  
  # Ping the AlterEgo API.
  # Returns either "PONG!" (indicating a successful connection to the API) or
  # an error.
  #
  # == Example
  #
  # AlterEgo.ping("valid_api_key")
  #
  def self.ping(key)
    call("#{API_URL}/check/ping.json?key=#{key}")
  end
  
  # Check an AlterEgo passcode.
  # Returns +true+ if the passcode if valid, and +false+ if the passcode is not
  # valid.
  #
  # == Example
  #
  # AlterEgo.password("valid_api_key", "passcode")
  #
  def self.password(key, passcode)
    call("#{API_URL}/check/password.json?key=#{key}&pass=#{passcode}")
  end
  
  private
  
  def self.call(url) # :nodoc:
    response = Net::HTTP.get_response(URI.parse(url))
    raise AlterEgo::APIError.new(response.body) if response.code.to_i == 500
    return nil if response.body == ''
    response.body
  end
  
  class APIError < StandardError
    def initialize(response) # :nodoc:
      error = JSON.parse(response)
      super "<#{error['code']}> #{error['name']}: #{error['message']}"
    end
  end
  
end
